@@ -46,7 +46,9 @@ class Tortoise:
         return cls._connections[connection_name]
 
     @classmethod
-    def describe_model(cls, model: Type[Model], serializable: bool = True) -> dict:
+    def describe_model(
+        cls, model: Type[Model], serializable: bool = True
+    ) -> dict:  # pragma: nocoverage
         """
         Describes the given list of models or ALL registered models.
 
@@ -57,184 +59,17 @@ class Tortoise:
             ``False`` if you want raw python objects,
             ``True`` for JSON-serialisable data. (Defaults to ``True``)
 
-        :return:
-            A dictionary containing the model description.
+        See :meth:`tortoise.models.Model.describe`
 
-            The base dict has a fixed set of keys that reference a list of fields
-            (or a single field in the case of the primary key):
-
-            .. code-block:: python3
-
-                {
-                    "name":                 str     # Qualified model name
-                    "app":                  str     # 'App' namespace
-                    "table":                str     # DB table name
-                    "abstract":             bool    # Is the model Abstract?
-                    "description":          str     # Description of table (nullable)
-                    "docstring":            str     # Model docstring (nullable)
-                    "unique_together":      [...]   # List of List containing field names that
-                                                    #  are unique together
-                    "pk_field":             {...}   # Primary key field
-                    "data_fields":          [...]   # Data fields
-                    "fk_fields":            [...]   # Foreign Key fields FROM this model
-                    "backward_fk_fields":   [...]   # Foreign Key fields TO this model
-                    "o2o_fields":           [...]   # OneToOne fields FROM this model
-                    "backward_o2o_fields":  [...]   # OneToOne fields TO this model
-                    "m2m_fields":           [...]   # Many-to-Many fields
-                }
-
-            Each field is specified as follows
-            (This assumes ``serializable=True``, which is the default):
-
-            .. code-block:: python3
-
-                {
-                    "name":         str     # Field name
-                    "field_type":   str     # Field type
-                    "db_column":    str     # Name of DB column
-                                            #  Optional: Only for pk/data fields
-                    "raw_field":    str     # Name of raw field of the Foreign Key
-                                            #  Optional: Only for Foreign Keys
-                    "db_field_types": dict  # DB Field types for default and DB overrides
-                    "python_type":  str     # Python type
-                    "generated":    bool    # Is the field generated by the DB?
-                    "nullable":     bool    # Is the column nullable?
-                    "unique":       bool    # Is the field unique?
-                    "indexed":      bool    # Is the field indexed?
-                    "default":      ...     # The default value (coerced to int/float/str/bool/null)
-                    "description":  str     # Description of the field (nullable)
-                    "docstring":    str     # Field docstring (nullable)
-                }
-
-            When ``serializable=False`` is specified some fields are not coerced to valid
-            JSON types. The changes are:
-
-            .. code-block:: python3
-
-                {
-                    "field_type":   Field   # The Field class used
-                    "python_type":  Type    # The actual Python type
-                    "default":      ...     # The default value as native type OR a callable
-                }
+        .. warning::
+           This is deprecated, please use :meth:`tortoise.models.Model.describe` instead
         """
-
-        def _type_name(typ: Type) -> str:
-            if typ.__module__ == "builtins":
-                return typ.__name__
-            if typ.__module__ == "typing":
-                return str(typ).replace("typing.", "")
-            return f"{typ.__module__}.{typ.__name__}"
-
-        def model_name(typ: Type[Model]) -> str:
-            name = typ._meta.table
-            for app in cls.apps.values():  # pragma: nobranch
-                for _name, _model in app.items():  # pragma: nobranch
-                    if typ == _model:
-                        name = _name
-            return f"{typ._meta.app}.{name}"
-
-        def type_name(typ: Any) -> Union[str, List[str]]:
-            try:
-                if issubclass(typ, Model):
-                    return model_name(typ)
-            except TypeError:
-                pass
-            try:
-                return _type_name(typ)
-            except AttributeError:
-                try:
-                    return [_type_name(_typ) for _typ in typ]  # pragma: nobranch
-                except TypeError:
-                    return str(typ)
-
-        def default_name(default: Any) -> Optional[Union[int, float, str, bool]]:
-            if isinstance(default, (int, float, str, bool, type(None))):
-                return default
-            if callable(default):
-                return f"<function {default.__module__}.{default.__name__}>"
-            return str(default)
-
-        def describe_field(name: str) -> dict:
-            # TODO: db_type
-            field = model._meta.fields_map[name]
-            field_type = getattr(field, "model_class", field.field_type)
-            desc = {
-                "name": name,
-                "field_type": field.__class__.__name__ if serializable else field.__class__,
-                "db_column": field.source_field or name,
-                "raw_field": None,
-                "db_field_types": field.get_db_field_types(),
-                "python_type": type_name(field_type) if serializable else field_type,
-                "generated": field.generated,
-                "nullable": field.null,
-                "unique": field.unique,
-                "indexed": field.index or field.unique,
-                "default": default_name(field.default) if serializable else field.default,
-                "description": field.description,
-                "docstring": field.docstring,
-                "constraints": field.constraints,
-            }
-
-            # Delete db fields for non-db fields
-            if not desc["db_field_types"]:
-                del desc["db_field_types"]
-
-            # Foreign Keys have
-            if isinstance(field, (ForeignKeyFieldInstance, OneToOneFieldInstance)):
-                del desc["db_column"]
-                desc["raw_field"] = field.source_field
-            else:
-                del desc["raw_field"]
-
-            # These fields are entierly "virtual", so no direct DB representation
-            if isinstance(
-                field, (ManyToManyFieldInstance, BackwardFKRelation, BackwardOneToOneRelation,),
-            ):
-                del desc["db_column"]
-
-            return desc
-
-        return {
-            "name": model_name(model),
-            "app": model._meta.app,
-            "table": model._meta.table,
-            "abstract": model._meta.abstract,
-            "description": model._meta.table_description or None,
-            "docstring": inspect.cleandoc(model.__doc__ or "") or None,
-            "unique_together": model._meta.unique_together or [],
-            "pk_field": describe_field(model._meta.pk_attr),
-            "data_fields": [
-                describe_field(name)
-                for name in model._meta.fields_map.keys()
-                if name != model._meta.pk_attr
-                and name in (model._meta.fields - model._meta.fetch_fields)
-            ],
-            "fk_fields": [
-                describe_field(name)
-                for name in model._meta.fields_map.keys()
-                if name in model._meta.fk_fields
-            ],
-            "backward_fk_fields": [
-                describe_field(name)
-                for name in model._meta.fields_map.keys()
-                if name in model._meta.backward_fk_fields
-            ],
-            "o2o_fields": [
-                describe_field(name)
-                for name in model._meta.fields_map.keys()
-                if name in model._meta.o2o_fields
-            ],
-            "backward_o2o_fields": [
-                describe_field(name)
-                for name in model._meta.fields_map.keys()
-                if name in model._meta.backward_o2o_fields
-            ],
-            "m2m_fields": [
-                describe_field(name)
-                for name in model._meta.fields_map.keys()
-                if name in model._meta.m2m_fields
-            ],
-        }
+        warnings.warn(
+            "Tortoise.describe_model(<MODEL>) is deprecated, please use <MODEL>.describe() instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return model.describe(serializable=serializable)
 
     @classmethod
     def describe_models(
@@ -269,8 +104,7 @@ class Tortoise:
                     models.append(model)
 
         return {
-            f"{model._meta.app}.{model.__name__}": cls.describe_model(model, serializable)
-            for model in models
+            f"{model._meta.app}.{model.__name__}": model.describe(serializable) for model in models
         }
 
     @classmethod
@@ -317,10 +151,8 @@ class Tortoise:
                 if model._meta._inited:
                     continue
                 model._meta._inited = True
-                if not model._meta.table:
-                    model._meta.table = model.__name__.lower()
-
-                pk_attr_changed = False
+                if not model._meta.db_table:
+                    model._meta.db_table = model.__name__.lower()
 
                 # TODO: refactor to share logic between FK & O2O
                 for field in model._meta.fk_fields:
@@ -348,6 +180,7 @@ class Tortoise:
                     else:
                         key_fk_object = deepcopy(related_model._meta.pk)
                         fk_object.to_field_instance = related_model._meta.pk
+                        fk_object.to_field = related_model._meta.pk_attr
 
                     key_field = f"{field}_id"
                     key_fk_object.pk = False
@@ -364,19 +197,23 @@ class Tortoise:
                         key_fk_object.source_field = key_field
                     model._meta.add_field(key_field, key_fk_object)
 
-                    fk_object.model_class = related_model
+                    fk_object.related_model = related_model
                     fk_object.source_field = key_field
                     backward_relation_name = fk_object.related_name
                     if backward_relation_name is not False:
                         if not backward_relation_name:
-                            backward_relation_name = f"{model._meta.table}s"
+                            backward_relation_name = f"{model._meta.db_table}s"
                         if backward_relation_name in related_model._meta.fields:
                             raise ConfigurationError(
                                 f'backward relation "{backward_relation_name}" duplicates in'
                                 f" model {related_model_name}"
                             )
                         fk_relation = BackwardFKRelation(
-                            model, f"{field}_id", fk_object.null, fk_object.description,
+                            model,
+                            f"{field}_id",
+                            key_fk_object.source_field,
+                            fk_object.null,
+                            fk_object.description,
                         )
                         fk_relation.to_field_instance = fk_object.to_field_instance
                         related_model._meta.add_field(backward_relation_name, fk_relation)
@@ -424,25 +261,28 @@ class Tortoise:
                         key_o2o_object.source_field = key_field
                     model._meta.add_field(key_field, key_o2o_object)
 
-                    o2o_object.model_class = related_model
+                    o2o_object.related_model = related_model
                     o2o_object.source_field = key_field
                     backward_relation_name = o2o_object.related_name
                     if backward_relation_name is not False:
                         if not backward_relation_name:
-                            backward_relation_name = f"{model._meta.table}"
+                            backward_relation_name = f"{model._meta.db_table}"
                         if backward_relation_name in related_model._meta.fields:
                             raise ConfigurationError(
                                 f'backward relation "{backward_relation_name}" duplicates in'
                                 f" model {related_model_name}"
                             )
                         o2o_relation = BackwardOneToOneRelation(
-                            model, f"{field}_id", null=True, description=o2o_object.description,
+                            model,
+                            f"{field}_id",
+                            key_o2o_object.source_field,
+                            null=True,
+                            description=o2o_object.description,
                         )
                         o2o_relation.to_field_instance = o2o_object.to_field_instance
                         related_model._meta.add_field(backward_relation_name, o2o_relation)
 
                     if o2o_object.pk:
-                        pk_attr_changed = True
                         model._meta.pk_attr = key_field
 
                 for field in list(model._meta.m2m_fields):
@@ -452,20 +292,22 @@ class Tortoise:
 
                     backward_key = m2m_object.backward_key
                     if not backward_key:
-                        backward_key = f"{model._meta.table}_id"
+                        backward_key = f"{model._meta.db_table}_id"
                         if backward_key == m2m_object.forward_key:
-                            backward_key = f"{model._meta.table}_rel_id"
+                            backward_key = f"{model._meta.db_table}_rel_id"
                         m2m_object.backward_key = backward_key
 
                     reference = m2m_object.model_name
                     related_app_name, related_model_name = split_reference(reference)
                     related_model = get_related_model(related_app_name, related_model_name)
 
-                    m2m_object.model_class = related_model
+                    m2m_object.related_model = related_model
 
                     backward_relation_name = m2m_object.related_name
                     if not backward_relation_name:
-                        backward_relation_name = m2m_object.related_name = f"{model._meta.table}s"
+                        backward_relation_name = (
+                            m2m_object.related_name
+                        ) = f"{model._meta.db_table}s"
                     if backward_relation_name in related_model._meta.fields:
                         raise ConfigurationError(
                             f'backward relation "{backward_relation_name}" duplicates in'
@@ -474,12 +316,12 @@ class Tortoise:
 
                     if not m2m_object.through:
                         related_model_table_name = (
-                            related_model._meta.table
-                            if related_model._meta.table
+                            related_model._meta.db_table
+                            if related_model._meta.db_table
                             else related_model.__name__.lower()
                         )
 
-                        m2m_object.through = f"{model._meta.table}_{related_model_table_name}"
+                        m2m_object.through = f"{model._meta.db_table}_{related_model_table_name}"
 
                     m2m_relation = ManyToManyFieldInstance(
                         f"{app_name}.{model_name}",
@@ -494,9 +336,6 @@ class Tortoise:
                     model._meta.filters.update(get_m2m_filters(field, m2m_object))
                     related_model._meta.add_field(backward_relation_name, m2m_relation)
 
-                if pk_attr_changed:
-                    model._meta.finalise_pk()
-
     @classmethod
     def _discover_client_class(cls, engine: str) -> BaseDBAsyncClient:
         # Let exception bubble up for transparency
@@ -527,7 +366,6 @@ class Tortoise:
                 if attr._meta.app and attr._meta.app != app_label:
                     continue
                 attr._meta.app = app_label
-                attr._meta.finalise_pk()
                 discovered_models.append(attr)
         if not discovered_models:
             warnings.warn(f'Module "{models_path}" has no models', RuntimeWarning, stacklevel=4)
@@ -615,8 +453,8 @@ class Tortoise:
         for app in cls.apps.values():
             for model in app.values():
                 model._meta.finalise_model()
-                model._meta.basetable = Table(model._meta.table)
-                model._meta.basequery = model._meta.db.query_class.from_(model._meta.table)
+                model._meta.basetable = Table(model._meta.db_table)
+                model._meta.basequery = model._meta.db.query_class.from_(model._meta.db_table)
                 model._meta.basequery_all_fields = model._meta.basequery.select(
                     *model._meta.db_fields
                 )
@@ -800,4 +638,4 @@ def run_async(coro: Coroutine) -> None:
         loop.run_until_complete(Tortoise.close_connections())
 
 
-__version__ = "0.16.4"
+__version__ = "0.16.6"
