@@ -338,7 +338,7 @@ class TruncationTestCase(SimpleTestCase):
         # TODO: This is a naive implementation: Will fail to clear M2M and non-cascade foreign keys
         for app in Tortoise.apps.values():
             for model in app.values():
-                await model._meta.db.execute_script(f"DELETE FROM {model._meta.table}")  # nosec
+                await model._meta.db.execute_script(f"DELETE FROM {model._meta.db_table}")  # nosec
 
 
 class TransactionTestContext:
