@@ -3,9 +3,18 @@ from functools import partial
 from typing import TYPE_CHECKING, Any, Dict, Iterable, Optional, Tuple
 
 from pypika import Table
-from pypika.enums import SqlTypes
-from pypika.functions import Cast, Upper
-from pypika.terms import Criterion, Term
+from pypika.functions import Upper
+from pypika.terms import (
+    BasicCriterion,
+    Criterion,
+    Enum,
+    Equality,
+    Term,
+    ValueWrapper,
+    basestring,
+    date,
+    format_quotes,
+)
 
 from tortoise.fields import Field
 from tortoise.fields.relational import BackwardFKRelation, ManyToManyFieldInstance
@@ -13,6 +22,71 @@ from tortoise.fields.relational import BackwardFKRelation, ManyToManyFieldInstan
 if TYPE_CHECKING:  # pragma: nocoverage
     from tortoise.models import Model
 
+##############################################################################
+# Here we monkey-patch PyPika Valuewrapper to behave differently for MySQL
+##############################################################################
+
+
+def get_value_sql(self, **kwargs):  # pragma: nocoverage
+    quote_char = kwargs.get("secondary_quote_char") or ""
+    dialect = kwargs.get("dialect")
+    if dialect:
+        dialect = dialect.value
+
+    if isinstance(self.value, Term):
+        return self.value.get_sql(**kwargs)
+    if isinstance(self.value, Enum):
+        return self.value.value
+    if isinstance(self.value, date):
+        value = self.value.isoformat()
+        return format_quotes(value, quote_char)
+    if isinstance(self.value, basestring):
+        value = self.value.replace(quote_char, quote_char * 2)
+        if dialect == "mysql":
+            value = value.replace("\\", "\\\\")
+        return format_quotes(value, quote_char)
+    if isinstance(self.value, bool):
+        return str.lower(str(self.value))
+    if self.value is None:
+        return "null"
+    return str(self.value)
+
+
+ValueWrapper.get_value_sql = get_value_sql
+##############################################################################
+
+
+class Like(BasicCriterion):  # type: ignore
+    def __init__(self, left, right, alias=None, escape=" ESCAPE '\\'") -> None:
+        """
+        A Like that supports an ESCAPE clause
+        """
+        super().__init__(" LIKE ", left, right, alias=alias)
+        self.escape = escape
+
+    def get_sql(self, quote_char='"', with_alias=False, **kwargs):
+        sql = "{left}{comparator}{right}{escape}".format(
+            comparator=self.comparator,
+            left=self.left.get_sql(quote_char=quote_char, **kwargs),
+            right=self.right.get_sql(quote_char=quote_char, **kwargs),
+            escape=self.escape,
+        )
+        if with_alias and self.alias:  # pragma: nocoverage
+            return '{sql} "{alias}"'.format(sql=sql, alias=self.alias)
+        return sql
+
+
+def escape_val(val: Any) -> Any:
+    if isinstance(val, str):
+        print(val)
+        return val.replace("\\", "\\\\")
+    return val
+
+
+def escape_like(val: str) -> str:
+    return val.replace("\\", "\\\\").replace("%", "\\%").replace("_", "\\_")
+
+
 ##############################################################################
 # Encoders
 # Should be type: (Any, instance: "Model", field: Field) -> type:
@@ -46,11 +120,17 @@ def string_encoder(value: Any, instance: "Model", field: Field) -> str:
 
 
 def is_in(field: Term, value: Any) -> Criterion:
-    return field.isin(value)
+    if value:
+        return field.isin(value)
+    # SQL has no False, so we return 1=0
+    return BasicCriterion(Equality.eq, ValueWrapper(1), ValueWrapper(0))
 
 
 def not_in(field: Term, value: Any) -> Criterion:
-    return field.notin(value) | field.isnull()
+    if value:
+        return field.notin(value) | field.isnull()
+    # SQL has no True, so we return 1=1
+    return BasicCriterion(Equality.eq, ValueWrapper(1), ValueWrapper(1))
 
 
 def between_and(field: Term, value: Tuple[Any, Any]) -> Criterion:
@@ -74,31 +154,31 @@ def not_null(field: Term, value: Any) -> Criterion:
 
 
 def contains(field: Term, value: str) -> Criterion:
-    return Cast(field, SqlTypes.VARCHAR).like(f"%{value}%")
+    return Like(field, field.wrap_constant(f"%{escape_like(value)}%"))
 
 
 def starts_with(field: Term, value: str) -> Criterion:
-    return Cast(field, SqlTypes.VARCHAR).like(f"{value}%")
+    return Like(field, field.wrap_constant(f"{escape_like(value)}%"))
 
 
 def ends_with(field: Term, value: str) -> Criterion:
-    return Cast(field, SqlTypes.VARCHAR).like(f"%{value}")
+    return Like(field, field.wrap_constant(f"%{escape_like(value)}"))
 
 
 def insensitive_exact(field: Term, value: str) -> Criterion:
-    return Upper(Cast(field, SqlTypes.VARCHAR)).eq(Upper(f"{value}"))
+    return Upper(field).eq(Upper(str(value)))
 
 
 def insensitive_contains(field: Term, value: str) -> Criterion:
-    return Upper(Cast(field, SqlTypes.VARCHAR)).like(Upper(f"%{value}%"))
+    return Like(Upper(field), field.wrap_constant(Upper(f"%{escape_like(value)}%")))
 
 
 def insensitive_starts_with(field: Term, value: str) -> Criterion:
-    return Upper(Cast(field, SqlTypes.VARCHAR)).like(Upper(f"{value}%"))
+    return Like(Upper(field), field.wrap_constant(Upper(f"{escape_like(value)}%")))
 
 
 def insensitive_ends_with(field: Term, value: str) -> Criterion:
-    return Upper(Cast(field, SqlTypes.VARCHAR)).like(Upper(f"%{value}"))
+    return Like(Upper(field), field.wrap_constant(Upper(f"%{escape_like(value)}")))
 
 
 ##############################################################################
@@ -107,7 +187,7 @@ def insensitive_ends_with(field: Term, value: str) -> Criterion:
 
 
 def get_m2m_filters(field_name: str, field: ManyToManyFieldInstance) -> Dict[str, dict]:
-    target_table_pk = field.model_class._meta.pk
+    target_table_pk = field.related_model._meta.pk
     return {
         field_name: {
             "field": field.forward_key,
@@ -141,34 +221,34 @@ def get_m2m_filters(field_name: str, field: ManyToManyFieldInstance) -> Dict[str
 
 
 def get_backward_fk_filters(field_name: str, field: BackwardFKRelation) -> Dict[str, dict]:
-    target_table_pk = field.model_class._meta.pk
+    target_table_pk = field.related_model._meta.pk
     return {
         field_name: {
-            "field": field.model_class._meta.pk_attr,
+            "field": field.related_model._meta.pk_attr,
             "backward_key": field.relation_field,
             "operator": operator.eq,
-            "table": Table(field.model_class._meta.table),
+            "table": Table(field.related_model._meta.db_table),
             "value_encoder": target_table_pk.to_db_value,
         },
         f"{field_name}__not": {
-            "field": field.model_class._meta.pk_attr,
+            "field": field.related_model._meta.pk_attr,
             "backward_key": field.relation_field,
             "operator": not_equal,
-            "table": Table(field.model_class._meta.table),
+            "table": Table(field.related_model._meta.db_table),
             "value_encoder": target_table_pk.to_db_value,
         },
         f"{field_name}__in": {
-            "field": field.model_class._meta.pk_attr,
+            "field": field.related_model._meta.pk_attr,
             "backward_key": field.relation_field,
             "operator": is_in,
-            "table": Table(field.model_class._meta.table),
+            "table": Table(field.related_model._meta.db_table),
             "value_encoder": partial(related_list_encoder, field=target_table_pk),
         },
         f"{field_name}__not_in": {
-            "field": field.model_class._meta.pk_attr,
+            "field": field.related_model._meta.pk_attr,
             "backward_key": field.relation_field,
             "operator": not_in,
-            "table": Table(field.model_class._meta.table),
+            "table": Table(field.related_model._meta.db_table),
             "value_encoder": partial(related_list_encoder, field=target_table_pk),
         },
     }
