@@ -9,6 +9,7 @@ from pypika import Order, Query, Table
 from tortoise.backends.base.client import BaseDBAsyncClient
 from tortoise.exceptions import (
     ConfigurationError,
+    IncompleteInstanceError,
     IntegrityError,
     OperationalError,
     TransactionManagementError,
@@ -21,6 +22,7 @@ from tortoise.fields.relational import (
     ForeignKeyFieldInstance,
     ManyToManyFieldInstance,
     ManyToManyRelation,
+    NoneAwaitable,
     OneToOneFieldInstance,
     ReverseRelation,
 )
@@ -33,25 +35,11 @@ MODEL = TypeVar("MODEL", bound="Model")
 # TODO: Define Filter type object. Possibly tuple?
 
 
-class _NoneAwaitable:
-    __slots__ = ()
-
-    def __await__(self) -> Generator[None, None, None]:
-        yield None
-
-    def __bool__(self) -> bool:
-        return False
-
-
-NoneAwaitable = _NoneAwaitable()
-
-
 def get_together(meta: "Model.Meta", together: str) -> Tuple[Tuple[str, ...], ...]:
     _together = getattr(meta, together, ())
 
-    if isinstance(_together, (list, tuple)):
-        if _together and isinstance(_together[0], str):
-            _together = (_together,)
+    if _together and isinstance(_together, (list, tuple)) and isinstance(_together[0], str):
+        _together = (_together,)
 
     # return without validation, validation will be done further in the code
     return _together
@@ -112,7 +100,7 @@ def _m2m_getter(
 ) -> ManyToManyRelation:
     val = getattr(self, _key, None)
     if val is None:
-        val = ManyToManyRelation(field_object.model_class, self, field_object)
+        val = ManyToManyRelation(self, field_object)
         setattr(self, _key, val)
     return val
 
@@ -151,7 +139,7 @@ def _get_comments(cls: "Type[Model]") -> Dict[str, str]:
 class MetaInfo:
     __slots__ = (
         "abstract",
-        "table",
+        "db_table",
         "app",
         "fields",
         "db_fields",
@@ -178,7 +166,7 @@ class MetaInfo:
         "_model",
         "table_description",
         "pk",
-        "db_pk_field",
+        "db_pk_column",
         "db_native_fields",
         "db_default_fields",
         "db_complex_fields",
@@ -188,7 +176,7 @@ class MetaInfo:
 
     def __init__(self, meta: "Model.Meta") -> None:
         self.abstract: bool = getattr(meta, "abstract", False)
-        self.table: str = getattr(meta, "table", "")
+        self.db_table: str = getattr(meta, "table", "")
         self.app: Optional[str] = getattr(meta, "app", None)
         self.unique_together: Tuple[Tuple[str, ...], ...] = get_together(meta, "unique_together")
         self.indexes: Tuple[Tuple[str, ...], ...] = get_together(meta, "indexes")
@@ -217,16 +205,21 @@ class MetaInfo:
         self._model: Type["Model"] = None  # type: ignore
         self.table_description: str = getattr(meta, "table_description", "")
         self.pk: Field = None  # type: ignore
-        self.db_pk_field: str = ""
+        self.db_pk_column: str = ""
         self.db_native_fields: List[Tuple[str, str, Field]] = []
         self.db_default_fields: List[Tuple[str, str, Field]] = []
         self.db_complex_fields: List[Tuple[str, str, Field]] = []
 
+    @property
+    def full_name(self) -> str:
+        return f"{self.app}.{self._model.__name__}"
+
     def add_field(self, name: str, value: Field) -> None:
         if name in self.fields_map:
             raise ConfigurationError(f"Field {name} already present in meta")
         value.model = self._model
         self.fields_map[name] = value
+        value.model_field_name = name
 
         if value.has_db_field:
             self.fields_db_projection[name] = value.source_field or name
@@ -264,10 +257,6 @@ class MetaInfo:
     def get_filter(self, key: str) -> dict:
         return self.filters[key]
 
-    def finalise_pk(self) -> None:
-        self.pk = self.fields_map[self.pk_attr]
-        self.db_pk_field = self.pk.source_field or self.pk_attr
-
     def finalise_model(self) -> None:
         """
         Finalise the model after it had been fully loaded.
@@ -318,7 +307,7 @@ class MetaInfo:
                     partial(
                         _fk_getter,
                         _key=_key,
-                        ftype=fk_field_object.model_class,
+                        ftype=fk_field_object.related_model,
                         relation_field=relation_field,
                         to_field=to_field,
                     ),
@@ -346,7 +335,7 @@ class MetaInfo:
                     partial(
                         _rfk_getter,
                         _key=_key,
-                        ftype=backward_fk_field_object.model_class,
+                        ftype=backward_fk_field_object.related_model,
                         frelfield=backward_fk_field_object.relation_field,
                         from_field=backward_fk_field_object.to_field_instance.model_field_name,
                     )
@@ -366,7 +355,7 @@ class MetaInfo:
                     partial(
                         _fk_getter,
                         _key=_key,
-                        ftype=o2o_field_object.model_class,
+                        ftype=o2o_field_object.related_model,
                         relation_field=relation_field,
                         to_field=to_field,
                     ),
@@ -396,7 +385,7 @@ class MetaInfo:
                     partial(
                         _ro2o_getter,
                         _key=_key,
-                        ftype=backward_o2o_field_object.model_class,
+                        ftype=backward_o2o_field_object.related_model,
                         frelfield=backward_o2o_field_object.relation_field,
                         from_field=backward_o2o_field_object.to_field_instance.model_field_name,
                     ),
@@ -507,7 +496,7 @@ class ModelMeta(type):
                         if custom_pk_present:
                             raise ConfigurationError(
                                 f"Can't create model {name} with two primary keys,"
-                                " only single pk are supported"
+                                " only single primary key is supported"
                             )
                         if value.generated and not value.allows_generated:
                             raise ConfigurationError(
@@ -573,6 +562,9 @@ class ModelMeta(type):
         meta.m2m_fields = m2m_fields
         meta.default_connection = None
         meta.pk_attr = pk_attr
+        meta.pk = fields_map.get(pk_attr)  # type: ignore
+        if meta.pk:
+            meta.db_pk_column = meta.pk.source_field or meta.pk_attr
         meta._inited = False
         if not fields_map:
             meta.abstract = True
@@ -606,9 +598,21 @@ class Model(metaclass=ModelMeta):
     def __init__(self, **kwargs: Any) -> None:
         # self._meta is a very common attribute lookup, lets cache it.
         meta = self._meta
+        self._partial = False
         self._saved_in_db = False
         self._custom_generated_pk = False
 
+        # Assign defaults for missing fields
+        for key in meta.fields.difference(self._set_kwargs(kwargs)):
+            field_object = meta.fields_map[key]
+            if callable(field_object.default):
+                setattr(self, key, field_object.default())
+            else:
+                setattr(self, key, field_object.default)
+
+    def _set_kwargs(self, kwargs: dict) -> Set[str]:
+        meta = self._meta
+
         # Assign values and do type conversions
         passed_fields = {*kwargs.keys()} | meta.fetch_fields
 
@@ -619,7 +623,7 @@ class Model(metaclass=ModelMeta):
                         f"You should first call .save() on {value} before referring to it"
                     )
                 setattr(self, key, value)
-                passed_fields.add(meta.fields_map[key].source_field)  # type: ignore
+                passed_fields.add(meta.fields_map[key].source_field)
             elif key in meta.fields_db_projection:
                 field_object = meta.fields_map[key]
                 if field_object.generated:
@@ -641,28 +645,36 @@ class Model(metaclass=ModelMeta):
                     "You can't set m2m relations through init, use m2m_manager instead"
                 )
 
-        # Assign defaults for missing fields
-        for key in meta.fields.difference(passed_fields):
-            field_object = meta.fields_map[key]
-            if callable(field_object.default):
-                setattr(self, key, field_object.default())
-            else:
-                setattr(self, key, field_object.default)
+        return passed_fields
 
     @classmethod
     def _init_from_db(cls: Type[MODEL], **kwargs: Any) -> MODEL:
         self = cls.__new__(cls)
+        self._partial = False
         self._saved_in_db = True
 
         meta = self._meta
 
-        for key, model_field, field in meta.db_native_fields:
-            setattr(self, model_field, kwargs[key])
-        for key, model_field, field in meta.db_default_fields:
-            value = kwargs[key]
-            setattr(self, model_field, None if value is None else field.field_type(value))
-        for key, model_field, field in meta.db_complex_fields:
-            setattr(self, model_field, field.to_python_value(kwargs[key]))
+        try:
+            # This is like so for performance reasons.
+            #  We want to avoid conditionals and calling .to_python_value()
+            # Native fields are fields that are already converted to/from python to DB type
+            #  by the DB driver
+            for key, model_field, field in meta.db_native_fields:
+                setattr(self, model_field, kwargs[key])
+            # Fields that don't override .to_python_value() are converted without a call
+            #  as we already know what we will be doing.
+            for key, model_field, field in meta.db_default_fields:
+                value = kwargs[key]
+                setattr(self, model_field, None if value is None else field.field_type(value))
+            # These fields need manual .to_python_value()
+            for key, model_field, field in meta.db_complex_fields:
+                setattr(self, model_field, field.to_python_value(kwargs[key]))
+        except KeyError:
+            self._partial = True
+            # TODO: Apply similar perf optimisation as above for partial
+            for key, value in kwargs.items():
+                setattr(self, key, meta.fields_map[key].to_python_value(value))
 
         return self
 
@@ -694,6 +706,24 @@ class Model(metaclass=ModelMeta):
     Can be used as a field name when doing filtering e.g. ``.filter(pk=...)`` etc...
     """
 
+    def update_from_dict(self, data: dict) -> MODEL:
+        """
+        Updates the current model with the provided dict.
+        This can allow mass-updating a model from a dict, also ensuring that datatype conversions happen.
+
+        This will ignore any extra fields, and NOT update the model with them,
+        but will raise errors on bad types or updating Many-instance relations.
+
+        :param data: The parameters you want to update in a dict format
+        :return: The current model instance
+
+        :raises ConfigurationError: When attempting to update a remote instance
+            (e.g. a reverse ForeignKey or ManyToMany relation)
+        :raises ValueError: When a passed parameter is not type compatible
+        """
+        self._set_kwargs(data)
+        return self  # type: ignore
+
     async def save(
         self,
         using_db: Optional[BaseDBAsyncClient] = None,
@@ -707,9 +737,26 @@ class Model(metaclass=ModelMeta):
             This is the subset of fields that should be updated.
             If the object needs to be created ``update_fields`` will be ignored.
         :param using_db: Specific DB connection to use instead of default bound
+
+        :raises IncompleteInstanceError: If the model is partial and the fields are not available for persistance.
         """
         db = using_db or self._meta.db
         executor = db.executor_class(model=self.__class__, db=db)
+        if self._partial:
+            if update_fields:
+                for field in update_fields:
+                    if not hasattr(self, self._meta.pk_attr):
+                        raise IncompleteInstanceError(
+                            f"{self.__class__.__name__} is a partial model without primary key fetchd. Partial update not available"
+                        )
+                    if not hasattr(self, field):
+                        raise IncompleteInstanceError(
+                            f"{self.__class__.__name__} is a partial model, field '{field}' is not available"
+                        )
+            else:
+                raise IncompleteInstanceError(
+                    f"{self.__class__.__name__} is a partial model, can only be saved with the relevant update_field provided"
+                )
         if self._saved_in_db:
             await executor.execute_update(self, update_fields)
         else:
@@ -754,9 +801,9 @@ class Model(metaclass=ModelMeta):
         Fetches the object if exists (filtering on the provided parameters),
         else creates an instance with any unspecified parameters as default values.
 
-        :param defaults:
+        :param defaults: Default values to be added to a created instance if it can't be fetched.
         :param using_db: Specific DB connection to use instead of default bound
-        :param kwargs:
+        :param kwargs: Query parameters.
         """
         if not defaults:
             defaults = {}
@@ -789,7 +836,7 @@ class Model(metaclass=ModelMeta):
             user = User(name="...", email="...")
             await user.save()
 
-        :param kwargs:
+        :param kwargs: Model parameters.
         """
         instance = cls(**kwargs)
         db = kwargs.get("using_db") or cls._meta.db
@@ -839,8 +886,8 @@ class Model(metaclass=ModelMeta):
         """
         Generates a QuerySet with the filter applied.
 
-        :param args:
-        :param kwargs:
+        :param args: Q funtions containing constraints. Will be AND'ed.
+        :param kwargs: Simple filter constraints.
         """
         return QuerySet(cls).filter(*args, **kwargs)
 
@@ -849,8 +896,8 @@ class Model(metaclass=ModelMeta):
         """
         Generates a QuerySet with the exclude applied.
 
-        :param args:
-        :param kwargs:
+        :param args: Q funtions containing constraints. Will be AND'ed.
+        :param kwargs: Simple filter constraints.
         """
         return QuerySet(cls).exclude(*args, **kwargs)
 
@@ -859,7 +906,7 @@ class Model(metaclass=ModelMeta):
         """
         Annotates the result set with extra Functions/Aggregations.
 
-        :param kwargs:
+        :param kwargs: Parameter name and the Function/Aggregation to annotate with.
         """
         return QuerySet(cls).annotate(**kwargs)
 
@@ -879,8 +926,8 @@ class Model(metaclass=ModelMeta):
 
             user = await User.get(username="foo")
 
-        :param args:
-        :param kwargs:
+        :param args: Q funtions containing constraints. Will be AND'ed.
+        :param kwargs: Simple filter constraints.
 
         :raises MultipleObjectsReturned: If provided search returned more than one object.
         :raises DoesNotExist: If object can not be found.
@@ -896,8 +943,8 @@ class Model(metaclass=ModelMeta):
 
             user = await User.get(username="foo")
 
-        :param args:
-        :param kwargs:
+        :param args: Q funtions containing constraints. Will be AND'ed.
+        :param kwargs: Simple filter constraints.
         """
         return QuerySet(cls).get_or_none(*args, **kwargs)
 
@@ -908,9 +955,9 @@ class Model(metaclass=ModelMeta):
         """
         Fetches related models for provided list of Model objects.
 
-        :param instance_list:
-        :param args:
-        :param using_db:
+        :param instance_list: List of Model objects to fetch relations for.
+        :param args: Relation names to fetch.
+        :param using_db: DO NOT USE
         """
         db = using_db or cls._meta.db
         await db.executor_class(model=cls, db=db).fetch_for_list(instance_list, *args)
@@ -956,6 +1003,84 @@ class Model(metaclass=ModelMeta):
                         " to ManyToMany field."
                     )
 
+    @classmethod
+    def describe(cls, serializable: bool = True) -> dict:
+        """
+        Describes the given list of models or ALL registered models.
+
+        :param serializable:
+            ``False`` if you want raw python objects,
+            ``True`` for JSON-serialisable data. (Defaults to ``True``)
+
+        :return:
+            A dictionary containing the model description.
+
+            The base dict has a fixed set of keys that reference a list of fields
+            (or a single field in the case of the primary key):
+
+            .. code-block:: python3
+
+                {
+                    "name":                 str     # Qualified model name
+                    "app":                  str     # 'App' namespace
+                    "table":                str     # DB table name
+                    "abstract":             bool    # Is the model Abstract?
+                    "description":          str     # Description of table (nullable)
+                    "docstring":            str     # Model docstring (nullable)
+                    "unique_together":      [...]   # List of List containing field names that
+                                                    #  are unique together
+                    "pk_field":             {...}   # Primary key field
+                    "data_fields":          [...]   # Data fields
+                    "fk_fields":            [...]   # Foreign Key fields FROM this model
+                    "backward_fk_fields":   [...]   # Foreign Key fields TO this model
+                    "o2o_fields":           [...]   # OneToOne fields FROM this model
+                    "backward_o2o_fields":  [...]   # OneToOne fields TO this model
+                    "m2m_fields":           [...]   # Many-to-Many fields
+                }
+
+            Each field is specified as defined in :meth:`tortoise.fields.base.Field.describe`
+        """
+        return {
+            "name": cls._meta.full_name,
+            "app": cls._meta.app,
+            "table": cls._meta.db_table,
+            "abstract": cls._meta.abstract,
+            "description": cls._meta.table_description or None,
+            "docstring": inspect.cleandoc(cls.__doc__ or "") or None,
+            "unique_together": cls._meta.unique_together or [],
+            "pk_field": cls._meta.pk.describe(serializable),
+            "data_fields": [
+                field.describe(serializable)
+                for name, field in cls._meta.fields_map.items()
+                if name != cls._meta.pk_attr and name in (cls._meta.fields - cls._meta.fetch_fields)
+            ],
+            "fk_fields": [
+                field.describe(serializable)
+                for name, field in cls._meta.fields_map.items()
+                if name in cls._meta.fk_fields
+            ],
+            "backward_fk_fields": [
+                field.describe(serializable)
+                for name, field in cls._meta.fields_map.items()
+                if name in cls._meta.backward_fk_fields
+            ],
+            "o2o_fields": [
+                field.describe(serializable)
+                for name, field in cls._meta.fields_map.items()
+                if name in cls._meta.o2o_fields
+            ],
+            "backward_o2o_fields": [
+                field.describe(serializable)
+                for name, field in cls._meta.fields_map.items()
+                if name in cls._meta.backward_o2o_fields
+            ],
+            "m2m_fields": [
+                field.describe(serializable)
+                for name, field in cls._meta.fields_map.items()
+                if name in cls._meta.m2m_fields
+            ],
+        }
+
     def __await__(self: MODEL) -> Generator[Any, None, MODEL]:
         async def _self() -> MODEL:
             return self
